#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
Created on 8 march 2024

@author: Vernay

Extraction / Backup of common Flow resources (date-dependant)  with Vortex

For each resource a "get" and "put" methods are provided with a number of arguments
that can specified by the user.

Mandatory arguments are :
    * `datebegin` and `dateend` of the perdio covered by the resource
    * `xpid` of the experiment that produced the resource
    * `geometry` of the resource

All other arguments are optional and defined by default with standard names.
However it is possible to use this tool in a more advanced way by precising :
    * the `namespace` to precise where to get/put the resource. Possible values are:
        - 'vortex.cache.fr'   : the local Vortex cache
        - 'vortex.archive.fr' : Hendrix Vortex cache
        - 'vortex.multi.fr'   : both the local and Hendrix caches
        - TODO : inclure sxcen
    * the 'filename' of the target resource (its name in the working directory)
    * the number of 'members' for ensemble simulations (TODO : utiliser la syntaxe X-Y-1)
    * the 'vapp' ('s2m' or 'edelweiss' currently) TODO : ajuster en fonction de la convention choisie
    * an 'abspath' to look for a file not in the current workind directory (TODO : not implemented yet)
    * the 'block' (the last directory(ies) names where the resource is stored on the Vortex cache)
    * the 'source_app' to precise which application produced the resource
    * the 'source_conf' to precise the configuration of the producing application
    * the 'model' to precise the name of the model that produced the resource (TODO : redondant ?)
      NB : the last 3 arguments are only used for resources at the begining of the processing chain

Examples :
==========

>>> from snowtools.scripts.extract.vortex import vortexIO

1. To save a 'PRO.nc' file present in the current working directory
 - covering the period 2024031306 --> 2024031406
 - produced by an experiment "XP00"
 - on a "GrandesRousses250m" geometry (already defined in the .vortexrc/geometries.ini file)

>>> vortexIO.put_pro('2024031306', '2024031406', 'XP00', "GrandesRousses250m")

2. To get the FORCING file generated by the S2M analysis dev chain of 13/03/2024 at 6h in the cor massif geometry:

>>> vortexIO.get_forcing('2024031206', '2024031306', 'nouveaux_guess@vernaym', 'cor', vapp='s2m', block='massifs',
        namebuild=None, date='2024031306', source_app='arpege', source_conf='4dvarfr')

3. [THEORETICAL EXAMPLE] To get a 16-member ensemble of pro files from a user "username"
 - covering the period yyyymmddhh --> YYYYMMDDHH
 - produced by an experiment "a_nice_experiment"
 - on a "new_geometry" geometry (must be defined in the .vortexrc/geometries.ini file)
and name each PRO file "THIS_IS_A_PRO.nc" :

>>> vortexIO.get_pro('yyyymmddhh', 'YYYYMMDDHH', 'a_nice_experiment@username', 'new_geometry',
        members=16, filename="THIS_IS_A_PRO.nc")

'''

import os
import vortex
import cen  # Import necessary to load vortex CEN-specific ressourees
from vortex import toolbox
from bronx.stdtypes.date import Date

import footprints

toolbox.active_now = True

t = vortex.ticket()

if 'MTOOLDIR' not in os.environ.keys():
    # Set a default MTOOLDIR to define the local Vortex cache
    os.environ['MTOOLDIR'] = os.path.join(os.environ['HOME'], 'cache')
    if not os.path.exists(os.environ['MTOOLDIR']):
        os.makedirs(os.environ['MTOOLDIR'])


def add_user(xpid):
    if '@' in xpid:
        return xpid
    else:
        user = os.environ['USER']
        return f'{xpid}@{user}'


def get(kind, datebegin, dateend, xpid, geometry, namespace, filename, nativefmt, members, vapp, namebuild, fatal,
        block=None, model=None, source_app=None, source_conf=None, date=None, abspath=None, scope=None, role=None,
        cutoff='assimilation'):
    """
    Main  method to fetch a Flow resurce with Vortex
    """

    storage = None
    if 'sxcen' in namespace:
        storage = 'sxcen.cnrm.meteo.fr'

    if scope == 'SesonalSnowCoverDiagnostic':
        # CEN's convention is to name period footprints 'datebegin' and 'dateend', but for SURFEX diagnostics
        # we use objects from common.data.diagnostics.SurfexPeriodDiagnostics with `begindate` and `enddate`
        # footprints...
        tb = toolbox.input(
            role           = role,
            kind           = kind,
            local          = f'mb[member]/{filename}' if members is not None else filename,
            nativefmt      = nativefmt,
            experiment     = add_user(xpid),
            geometry       = geometry,
            begindate      = Date(datebegin),
            enddate        = Date(dateend),
            vapp           = vapp,
            vconf          = '[geometry:tag]',
            model          = model,
            cutoff         = cutoff,
            source_app     = source_app,
            source_conf    = source_conf,
            namespace      = namespace,
            storage        = storage,
            block          = block,
            member         = None if members is None else footprints.util.rangex(0, members - 1),
            fatal          = fatal
        )
        return tb
    else:
        tb = toolbox.input(
            role           = role,
            kind           = kind,
            local          = f'mb[member]/{filename}' if members is not None else filename,
            nativefmt      = nativefmt,
            experiment     = add_user(xpid),
            geometry       = geometry,
            date           = Date(dateend) if date is None else date,
            datebegin      = Date(datebegin),
            dateend        = Date(dateend),
            vapp           = vapp,
            vconf          = '[geometry:tag]',
            model          = model,
            cutoff         = cutoff,
            source_app     = source_app,
            source_conf    = source_conf,
            namespace      = namespace,
            storage        = storage,
            block          = block,
            member         = None if members is None else footprints.util.rangex(0, members - 1),
            fatal          = fatal
        )
        return tb


def put(kind, datebegin, dateend, xpid, geometry, namespace, filename, nativefmt, members, vapp, namebuild, fatal,
        block=None, model=None, source_app=None, source_conf=None, date=None, abspath=None, scope=None, role=None,
        cutoff=None):
    """
    Main  method to save a Flow resurce with Vortex
    """

    storage = None
    if 'sxcen' in namespace:
        storage = 'sxcen.cnrm.meteo.fr'

    if scope == 'SesonalSnowCoverDiagnostic':
        # CEN's convention is to name period footprints 'datebegin' and 'dateend', but for SURFEX diagnostics
        # we use objects from common.data.diagnostics.SurfexPeriodDiagnostics with `begindate` and `enddate`
        # footprints...
        tb = toolbox.output(
            role           = role,
            kind           = kind,
            local          = f'mb[member]/{filename}' if members is not None else filename,
            nativefmt      = nativefmt,
            experiment     = add_user(xpid),
            geometry       = geometry,
            begindate      = Date(datebegin),
            enddate        = Date(dateend),
            vapp           = vapp,
            vconf          = '[geometry:tag]',
            model          = model,
            cutoff         = cutoff,
            source_app     = source_app,
            source_conf    = source_conf,
            namespace      = namespace,
            storage        = storage,
            block          = block,
            member         = None if members is None else footprints.util.rangex(0, members - 1),
            fatal          = fatal
        )
        return tb
    else:
        tb = toolbox.output(
            role           = role,
            kind           = kind,
            local          = f'mb[member]/{filename}' if members is not None else filename,
            nativefmt      = nativefmt,
            experiment     = add_user(xpid),
            geometry       = geometry,
            date           = Date(dateend) if date is None else date,
            datebegin      = Date(datebegin),
            dateend        = Date(dateend),
            vapp           = vapp,
            vconf          = '[geometry:tag]',
            model          = model,
            cutoff         = cutoff,
            source_app     = source_app,
            source_conf    = source_conf,
            namespace      = namespace,
            storage        = storage,
            block          = block,
            member         = None if members is None else footprints.util.rangex(0, members - 1),
            fatal          = fatal
        )
        return tb


def get_pro(datebegin, dateend, xpid, geometry, namespace='vortex.multi.fr', filename='PRO.nc',
            members=None, vapp='s2m', abspath=None, block='pro', date=None, namebuild='flat@cen', fatal=True):

    # PRO-specific footprints
    kind      = 'SnowpackSimulation'
    model     = 'surfex'
    nativefmt = 'netcdf'

    pro = get(kind, datebegin, dateend, xpid, geometry, namespace, filename, nativefmt, members, vapp,
              namebuild, fatal, block=block, abspath=abspath, date=date, model=model)

    print(t.prompt, 'PRO input =', pro)
    print()


def put_pro(datebegin, dateend, xpid, geometry, namespace='vortex.multi.fr', filename='PRO.nc',
            members=None, vapp='s2m', abspath=None, block='pro', date=None, namebuild='flat@cen', fatal=True):

    # PRO-specific footprints
    kind      = 'SnowpackSimulation'
    model     = 'surfex'
    nativefmt = 'netcdf'

    pro = put(kind, datebegin, dateend, xpid, geometry, namespace, filename, nativefmt, members, vapp,
              namebuild, fatal, block=block, abspath=abspath, date=date, model=model)

    print(t.prompt, 'PRO output =', pro)
    print()


def get_diag(datebegin, dateend, xpid, geometry, namespace='vortex.multi.fr', filename='DIAG.nc',
             members=None, vapp='s2m', block='diag', abspath=None, namebuild='flat@cen', fatal=True):

    # DIAG-specific footprints
    kind      = 'diagnostics'
    scope     = 'SesonalSnowCoverDiagnostic'
    nativefmt = 'netcdf'
    model     = 'surfex'

    diag = get(kind, datebegin, dateend, xpid, geometry, namespace, filename, nativefmt, members, vapp,
               namebuild, fatal, block=block, abspath=abspath, model=model, scope=scope)

    print(t.prompt, 'DIAG input =', diag)
    print()


def put_diag(datebegin, dateend, xpid, geometry, namespace='vortex.multi.fr', filename='DIAG.nc',
             members=None, vapp='s2m', block='diag', abspath=None, namebuild='flat@cen', fatal=True):

    # DIAG-specific footprints
    kind      = 'diagnostics'
    scope     = 'SesonalSnowCoverDiagnostic'
    nativefmt = 'netcdf'
    model     = 'surfex'

    diag = put(kind, datebegin, dateend, xpid, geometry, namespace, filename, nativefmt, members, vapp,
               namebuild, fatal, block=block, abspath=abspath, model=model, scope=scope)

    print(t.prompt, 'DIAG ouput =', diag)
    print()


def get_forcing(datebegin, dateend, xpid, geometry, namespace='vortex.multi.fr', filename='FORCING.nc', model='safran',
                members=None, vapp='s2m', block='meteo', abspath=None, source_app=None, source_conf=None,
                date=None, namebuild='flat@cen', fatal=True, cutoff='assimilation'):

    # FORCING-specific footprints
    role      = 'Forcing'
    kind      = 'MeteorologicalForcing'
    nativefmt = 'netcdf'

    forcing = get(kind, datebegin, dateend, xpid, geometry, namespace, filename, nativefmt, members, vapp,
                  namebuild, fatal, block=block, abspath=abspath, source_app=source_app, source_conf=source_conf,
                  date=date, model=model, role=role, cutoff=cutoff)

    print(t.prompt, 'FORCING input =', forcing)
    print()


def put_forcing(datebegin, dateend, xpid, geometry, namespace='vortex.multi.fr', filename='FORCING.nc', model='safran',
                members=None, vapp='s2m', block='meteo', abspath=None, source_app=None, source_conf=None,
                date=None, namebuild='flat@cen', fatal=True, cutoff='assimilation'):

    # FORCING-specific footprints
    role      = 'Forcing'
    kind      = 'MeteorologicalForcing'
    nativefmt = 'netcdf'

    forcing = put(kind, datebegin, dateend, xpid, geometry, namespace, filename, nativefmt, members, vapp,
                  namebuild, fatal, block=block, abspath=abspath, source_app=source_app, source_conf=source_conf,
                  date=date, model=model, role=role, cutoff=cutoff)

    print(t.prompt, 'FORCING output =', forcing)
    print()


def get_precipitation(datebegin, dateend, xpid, geometry, namespace='vortex.multi.fr', filename='PRECIPITATION.nc',
                      members=None, vapp='s2m', block='', abspath=None, source_app=None, source_conf=None,
                      model=None, date=None, namebuild='flat@cen', fatal=True):

    # Precipitation-specific footprints
    role      = 'Precipitation'
    kind      = 'Precipitation'
    nativefmt = 'netcdf'

    precipitation = get(kind, datebegin, dateend, xpid, geometry, namespace, filename, nativefmt, members, vapp,
                        namebuild, fatal, block=block, abspath=abspath, source_app=source_app, source_conf=source_conf,
                        date=date, model=model, role=role)

    print(t.prompt, 'PRECIPITATION input =', precipitation)
    print()


def put_precipitation(datebegin, dateend, xpid, geometry, namespace='vortex.multi.fr', filename='PRECIPITATION.nc',
                      members=None, vapp='s2m', block='', abspath=None, source_app=None, source_conf=None,
                      model=None, date=None, namebuild='flat@cen', fatal=True):

    # Precipitation-specific footprints
    role      = 'Precipitation'
    kind      = 'Precipitation'
    nativefmt = 'netcdf'

    precipitation = put(kind, datebegin, dateend, xpid, geometry, namespace, filename, nativefmt, members, vapp,
                        namebuild, fatal, block=block, abspath=abspath, source_app=source_app, source_conf=source_conf,
                        date=date, model=model, role=role)

    print(t.prompt, 'PRECIPITATION output =', precipitation)
    print()


def get_wind(datebegin, dateend, xpid, geometry, namespace='vortex.multi.fr', filename='WIND.nc',
             members=None, vapp='s2m', block='', abspath=None, source_app='arome', source_conf='devine',
             model='devine', date=None, namebuild='flat@cen', fatal=True):

    # Wind-specific footprints
    role      = 'Wind'
    kind      = 'Wind'
    nativefmt = 'netcdf'

    wind = get(kind, datebegin, dateend, xpid, geometry, namespace, filename, nativefmt, members, vapp,
               namebuild, fatal, block=block, abspath=abspath, source_app=source_app, source_conf=source_conf,
               date=date, model=model, role=role)

    print(t.prompt, 'WIND input =', wind)
    print()


def put_wind(datebegin, dateend, xpid, geometry, namespace='vortex.multi.fr', filename='WIND.nc',
             members=None, vapp='s2m', block='', abspath=None, source_app='arome', source_conf='devine',
             model='devine', date=None, namebuild='flat@cen', fatal=True):

    # Wind-specific footprints
    role      = 'Wind'
    kind      = 'Wind'
    nativefmt = 'netcdf'

    wind = put(kind, datebegin, dateend, xpid, geometry, namespace, filename, nativefmt, members, vapp,
               namebuild, fatal, block=block, abspath=abspath, source_app=source_app, source_conf=source_conf,
               date=date, model=model, role=role)

    print(t.prompt, 'WIND output =', wind)
    print()
